#! /usr/local/bin/python
#-*- coding: utf-8 -*-

#
# This script generates the base of hash values.
#

from __future__ import with_statement

import hashlib
import pickle
import os
import glob
import re
import rsa


# Files to scan
number_of_files_to_scan = 3
file0 = "/etc/cron.hourly/hids"
file1 = "/boot/grub/menu.lst"
file2 = "/root/lighttpd.sh"



def search_files(motif, root_path):
    """Return a list of files.

    Search fo files containing 'motif' that
    aren't symbolic links.
    """
    result = []
    w = os.walk(root_path)
    for (path, dirs, files) in w:
        for f in files:
            if re.compile(motif).search(f):
		# if not a symbolic link
		if not os.path.islink(os.path.join(path, f)):
    	            result.append(os.path.join(path, f))
    return result


def hash_file(target_file):
    """Hash the file given in parameter.
    """
    sha1_hash = hashlib.sha1()
    opened_file = None
    hashed_data = None

    # Handle the errors that may happen
    try:
        opened_file = open(target_file, "r")
        data = opened_file.read()
    except:
        # The specified file does not exist,
        # remove from the list.
        print "File", target_file, "does not exist."
        globals()['number_of_files_to_scan'] = \
            globals()['number_of_files_to_scan'] - 1
        del list_of_files[list_of_files.index(target_file)]
    finally:
        if opened_file is not None:
            opened_file.close()

    if opened_file is not None:
        sha1_hash.update(data)
        hashed_data = sha1_hash.hexdigest()

    return hashed_data



if __name__ == '__main__':
    # Point of entry in execution mode.
    print "Loading public key"

    # address of the saved base of hash values :
    base_location = "/home/cdjs/pyHIDS/base"
    # address of the public key
    pub_key_location = "/home/cdjs/pyHIDS/cle_pub"

    # Load the public key to crypt the result.
    with open(pub_key_location, "r") as key:
        pub_key = pickle.load(key)

    # Open the base of hash values
    try:
        base = open(base_location, "w")
    except Exception , e:
        print "Error :", e
        exit(0)

    # Parse the files to scan in a list
    list_of_files = []
    for i in range(number_of_files_to_scan):
        list_of_files.append(globals()["file"+str(i)])




    # adding the configuration files in /etc
    list_of_files.extend(search_files("conf", "/etc"))
    
    # adding the files of the HIDS
    list_of_files.extend(search_files("py", "/home/cdjs/pyHIDS"))

    # adding the files of pyLock
    list_of_files.extend(search_files(".", "/home/cdjs/pyLock"))

    # site files
    list_of_files.extend(search_files("php", "/home/www-data"))
    list_of_files.extend(search_files("php", "/home/www-data2"))
    list_of_files.extend(search_files("conf", "/home/www-data"))
    list_of_files.extend(search_files("conf", "/home/www-data2"))
    
    number_of_files_to_scan = len(list_of_files)




    print "Generating data base..."
    # Hash and write each files
    for file in list_of_files:
        hash_value = hash_file(file)
        if hash_value is not None:
            line = file + ":" + hash_value + ":"
            encrypted_line = rsa.encrypt(line, pub_key)
            encrypted_line = encrypted_line.replace('\n', '')
            base.write(encrypted_line+"\n")
    print number_of_files_to_scan, "files in the base."
    base.close()
